#include <QApplication>
#include <QDebug>
#include <QMainWindow>
#include <QDockWidget>
#include <QTextBrowser>
#include <QMouseEvent>
#include <QMetaObject>

#include <iterator>

static bool optLogMouseMove = false;

class EventFilter : public QObject
{
public:
    explicit EventFilter(QObject *parent = 0) : QObject(parent) {}

    bool eventFilter(QObject *watched, QEvent *event) override;
};

static void formatEvent(QObject *watched, QEvent *event)
{
    static int n = 0;
    QDebug d = qDebug();
    d.noquote();
    d.nospace();
    d << '#' << n++ << ' ';
    QString on = watched->objectName();
    if (!on.isEmpty())
        d << '"' << on << "\"/";
    d << watched->metaObject()->className() << ' ' << event;
}

bool EventFilter::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type()) {
    case QEvent::MouseMove:
    case QEvent::NonClientAreaMouseMove:
        if (optLogMouseMove)
            formatEvent(watched, event);
        break;
    case QEvent::NonClientAreaMouseButtonPress:
    case QEvent::NonClientAreaMouseButtonRelease:
    case QEvent::NonClientAreaMouseButtonDblClick:
    case QEvent::MouseButtonPress:
    case QEvent::MouseButtonRelease:
    case QEvent::MouseButtonDblClick:
    case QEvent::HoverMove:
    case QEvent::HoverLeave:
    case QEvent::HoverEnter:
    case QEvent::DragMove:
    case QEvent::DragEnter:
    case QEvent::DragResponse:
    case QEvent::DragLeave:
    case QEvent::FocusIn:
    case QEvent::FocusOut:
    case QEvent::FocusAboutToChange:
        formatEvent(watched, event);
        break;
    default:
        break;
    }
   return QObject::eventFilter(watched, event);
}


class MainWindow : public QMainWindow
{
public:
    explicit MainWindow(QWidget *parent = 0);
};

MainWindow::MainWindow(QWidget *)
{
    QDockWidget *dw =  new QDockWidget("This is my Title");
    dw->setObjectName("Dock");
    addDockWidget(Qt::LeftDockWidgetArea, dw);
    auto tb = new QTextBrowser;
    tb->setObjectName("TextBrowser");
    setCentralWidget(tb);
    setObjectName("Main");
}

int main(int argc, char *argv[])
{
    QStringList args;
    std::copy(argv + 1, argv + argc,
              std::back_inserter(args));
    QApplication a(argc, argv);
    optLogMouseMove = args.contains("-m");
    if (!args.contains("-e"))
        a.installEventFilter(new EventFilter(&a));
    MainWindow w;
    QString t = QLatin1String(QT_VERSION_STR) + QLatin1Char(' ') + a.platformName();
    w.setWindowTitle(t);
    qDebug() << t << args;
    w.show();

    return a.exec();
}

